=begin
Copyright 2009, Matt666
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   		Free

Author:     	Matthieu NOBLET
Organization:   Matt666
Name:      		ExtendTool
Version:   		1.1
SU Version:   	6 and 7
Date:      		16/06/2009
Description:   	Extend pointed edge. You can select limits first. 
				If nothing is selected before running tool, script select all the model as limit.
Usage:      	Menu "plugins/Extend edges".
				Type "Matt_Edges.extend" in the ruby console to load it manually.
History:
	1.100	26/06/2009
			New extended edges broke edges of the limits. So limits changed while script worked.
			
	1.000   16/06/2009
			First version
=end

require 'sketchup.rb'

if !file_loaded?('TtMn_selection') then
	if $submenu
		organizer = $submenu
	else
		organizer = UI.menu('Plugins')
	end
	$TtMn_selection_plugins_submenu = organizer.add_submenu('Selection')
	UI.add_context_menu_handler do |menu|
		$TtMn_selection_contextual_submenu=menu.add_submenu('Selection')
	end
	file_loaded('TtMn_selection')
end

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
	organizer.add_item('Extend edges'){ Matt_Edges.extend }
	UI.add_context_menu_handler do |menu|
	$TtMn_selection_contextual_submenu.add_item('Extend edges'){ Matt_Edges.extend } if Matt_Edges.extend_valid?
	end
end


module Matt_Edges

def self.extend
	var=false
	model=Sketchup.active_model
	sel=model.selection
	puts '\nNo selection. All the model is the limit.' if sel.empty?
	sel.each{|e|var=true if (not e.is_a?(Sketchup::Face)) and (not e.is_a?(Sketchup::Edge))}
	var ? UI.messagebox('Please select only edges or faces.') : model.select_tool(ExtendTool.new(sel.to_a,0))
end

def self.extend_valid?
	ss = Sketchup.active_model.selection
	return false if ss.empty?
	ss.each{|e|return false if !e.is_a?(Sketchup::Edge) and !e.is_a?(Sketchup::Face)}
	return true
end

class ExtendTool
	@@cursor = 0
	def initialize(sel,int)
		@@cursor = UI::create_cursor((File.join(File.dirname(__FILE__), "Matt_ExtendTool","ExtendTool_cursor.png")),10,10) if @@cursor == 0
		@sel=sel
		@pt_lst=[]
		sel.each do |e|
			if e.is_a?(Sketchup::Edge)
				d=e.start.position.distance e.end.position
				v=e.line[1]
				v.length=d
				@pt_lst.push [e.start.position,v]
			elsif e.is_a?(Sketchup::Face)
				@pt_lst.push e.plane
			end
		end
		@int=int
		@ip=nil
		@ip1=nil
		
	end
	
	def onSetCursor
		UI::set_cursor(@@cursor)
	end
	
	def activate
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		model=Sketchup.active_model
		msg='Extend edges'
		Sketchup.version.to_i>=7 ? model.start_operation(msg,true) : model.start_operation(msg)
		reset(nil)
	end
	
	def deactivate(view)
		view.invalidate
		@pt_lst=nil
		@sel=nil
		Sketchup.active_model.commit_operation
		Sketchup.send_action("selectSelectionTool:")
	end
	
	def onCancel(flag, view)
		reset(view)
	end
	
	def reset(view)
		@ip1.clear
		if view
			view.tooltip=nil
			view.invalidate
		end
		@state=0
		Sketchup::set_status_text("Select an edge to extend:")
		
	end
	
	def onMouseMove(flags, x, y, view)
		@ip.pick view, x, y
		if @ip!=@ip1
			@ip1.copy! @ip
			view.invalidate if @ip1.display?
			if ["On Edge", "Midpoint", "Endpoint"].include?(@ip1.tooltip)
				view.tooltip = "Select an Edge"
			else
				view.tooltip = "Not an edge"
			end
		end
		
	end
	
	def onLButtonDown(flags, x, y, view)
		@ip1.pick view, x, y
		if @ip1.valid?
			@ent=nil
			view.lock_inference
			if ["On Edge","Midpoint"].include?(@ip1.tooltip)
				@ent=@ip1.edge
			elsif @ip1.tooltip == "Endpoint"
				for e in @ip1.vertex.edges
					@ent=e if !@sel.to_a.include?(e)
				end
			end	
			@state=1
		end
	end
	
	def onLButtonUp(flags, x, y, view)
		if @ent.is_a?(Sketchup::Edge) and !@sel.to_a.include?(@ent)
			if @sel.length==0
				s=Sketchup.active_model.active_entities.select{|e|(e.is_a?(Sketchup::Edge) or e.is_a?(Sketchup::Face)) and @ent!=e}
				create_geometry(@ent, s, view)
			else
				create_geometry(@ent, @pt_lst, view)
			end
		end
	end
	
	def draw(view)
	    @ip1.draw(view) if @ip1.valid? and @ip1.display?
	end
	
	def create_geometry(e, s, view)
		lst=[]
		p=nil
		model=Sketchup.active_model
		ents=model.active_entities
		p1=e.start.position
		p2=e.end.position
		s.each{|limit|search_for_inters(limit,e,lst)}
		if lst.length>0
			lst=lst.uniq.sort{|x,y| x[1]<=>y[1]}
			# Point sous la souris
			ip1=Geom::Point3d.new(@ip1.position)
			# Trouver l'extrmit la plus loigne de ip1
			p1.distance(ip1)<p2.distance(ip1) ? pbase=p2 : pbase=p1
			# Crer les lignes
			if pdest=first_point(lst,pbase,ip1)
				ents.add_line(pbase,pdest)
			end
		else
			puts "No limit found..."
		end
		view.invalidate
		reset view
	end
	
	# Trouver le point d'origine de la ligne
	def first_point(lst,ps,pm)
		lst.each{|elt|return elt[0] if pm.between_points?(ps,elt[0])}
		return false
	end
	
	# Trouver le point d'intersection entre 2 lignes ou 1 ligne et 1 plan
	def search_for_inters(lim,ent,l)
		p1=ent.start.position
		p2=ent.end.position
		if lim.length==4 and p1!=p2 
			inters=Geom.intersect_line_plane(ent.line, lim)
			if inters and p1&&p2!=inters and p2.distance(inters)&&p1.distance(inters)!=0.0
				l.push([inters, p2.distance(inters),p1.distance(inters)]) 
			end
		elsif lim.length==2 and p1!=p2 
			inters=Geom.intersect_line_line(ent.line, lim)
			if inters and p2.distance(inters)&&p1.distance(inters)!=0.0 and p1&&p2!=inters and !inters.between_points?(p1,p2)
				if @int==0
					l.push([inters, p2.distance(inters),p1.distance(inters)]) if inters.between_points?(lim[0],lim[0].offset(lim[1]))
				else
					l.push([inters, p2.distance(inters),p1.distance(inters)]) 
				end
			end
		end
	end
	
end #Class

end #Module

class Geom::Point3d
	def between_points?(p1,p2)
		if p1.is_a?(Geom::Point3d) and p2.is_a?(Geom::Point3d)
			return true if p==p2 or p==p1
			v1=p1.vector_to self
			v2=self.vector_to p2
			return false if v1.length==0 or v2.length==0
			v1.length=v2.length=1
			return true if v1==v2
			return false
		else
			puts "Error in 'between_points?': #<ClassError: p1 or p2 are not Point3d...>"
		end
	end
end #Class
